"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.Agent = void 0;
var os_1 = __importDefault(require("os"));
var isType_1 = require("./utils/isType");
var telemetry_1 = require("./telemetry");
var Metadata_1 = require("./Metadata");
var Environment_1 = require("./Environment");
var Transmitter_1 = require("./Transmitter");
var agentHelpers_1 = require("./agentHelpers");
var version_1 = require("./version");
var Agent = (function () {
    function Agent(options) {
        this.environment = new Environment_1.Environment();
        this.metadata = new Metadata_1.Metadata();
        this.telemetry = new telemetry_1.TelemetryBuffer(30);
        this._onErrorFns = [];
        this.options = Object.assign({}, Agent.defaults, options);
        this.options.correlationId = this.options.correlationId;
        if ((0, isType_1.isFunction)(options.onError)) {
            this.onError(options.onError);
            delete this.options.onError;
        }
        this.metadata = new Metadata_1.Metadata(this.options.metadata);
        delete this.options.metadata;
        if (this.options.defaultMetadata) {
            this.metadata.add("hostname", os_1["default"].hostname());
            this.metadata.add("username", os_1["default"].userInfo().username);
            this.metadata.add("cwd", process.cwd());
            if (process.mainModule) {
                this.metadata.add("filename", process.mainModule.filename);
            }
        }
    }
    Agent.prototype.captureError = function (error, entry) {
        if (error["__trackjs__"]) {
            return false;
        }
        if (!(error instanceof Error)) {
            error = new Error("" + error);
        }
        Object.defineProperty(error, "__trackjs__", {
            value: true,
            enumerable: false
        });
        var report = this.createErrorReport(error, entry);
        var hasIgnored = false;
        __spreadArray([agentHelpers_1.deduplicate, agentHelpers_1.truncate], this._onErrorFns, true).forEach(function (fn) {
            if (!hasIgnored) {
                try {
                    hasIgnored = !fn(report);
                }
                catch (e) {
                    report.console.push({
                        severity: "error",
                        message: "Your TrackJS onError handler failed: " + e.message,
                        timestamp: new Date().toISOString()
                    });
                }
            }
        });
        if (hasIgnored) {
            return false;
        }
        this.telemetry.add("c", new telemetry_1.ConsoleTelemetry("error", [error]));
        (0, Transmitter_1.transmit)({
            url: this.options.errorURL,
            method: "POST",
            queryParams: {
                token: this.options.token,
                v: version_1.RELEASE_VERSION
            },
            payload: report
        });
        error["TrackJS"] = {
            correlationId: report.customer.correlationId,
            entry: report.entry,
            url: "https://my.trackjs.com/details/correlationid/" + report.customer.correlationId
        };
        return true;
    };
    Agent.prototype.captureUsage = function () {
        (0, Transmitter_1.transmit)({
            url: this.options.usageURL,
            method: "GET",
            queryParams: {
                token: this.options.token,
                correlationId: this.options.correlationId,
                application: this.options.application
            }
        });
    };
    Agent.prototype.clone = function (options) {
        var cloned = new Agent(Object.assign({}, this.options, options));
        cloned.metadata = this.metadata.clone();
        cloned.telemetry = this.telemetry.clone();
        cloned.environment = this.environment.clone();
        this._onErrorFns.forEach(function (fn) { return cloned.onError(fn); });
        if (options && options.metadata) {
            cloned.metadata.add(options.metadata);
        }
        return cloned;
    };
    Agent.prototype.configure = function (options) {
        this.options = Object.assign(this.options, options);
    };
    Agent.prototype.createErrorReport = function (error, entry) {
        var now = new Date();
        return {
            agentPlatform: "node",
            bindStack: null,
            bindTime: null,
            console: this.telemetry.getAllByCategory("c"),
            customer: {
                application: this.options.application,
                correlationId: this.options.correlationId,
                sessionId: this.options.sessionId,
                token: this.options.token,
                userId: this.options.userId,
                version: this.options.version
            },
            entry: entry,
            environment: {
                age: now.getTime() - this.environment.start.getTime(),
                dependencies: this.options.dependencies ? this.environment.getDependencies() : {},
                originalUrl: this.environment.url,
                referrer: this.environment.referrerUrl,
                userAgent: this.environment.userAgent
            },
            file: "",
            message: error.message,
            metadata: this.metadata.get(),
            nav: [],
            network: this.telemetry.getAllByCategory("n"),
            url: this.environment.url,
            stack: error.stack,
            throttled: 0,
            timestamp: now.toISOString(),
            visitor: [],
            version: version_1.RELEASE_VERSION
        };
    };
    Agent.prototype.onError = function (func) {
        this._onErrorFns.push(func);
    };
    Agent.defaults = {
        token: "",
        application: "",
        defaultMetadata: true,
        dependencies: true,
        errorURL: "https://capture.trackjs.com/capture/node",
        faultURL: "https://usage.trackjs.com/fault.gif",
        sessionId: "",
        usageURL: "https://usage.trackjs.com/usage.gif",
        userId: "",
        version: ""
    };
    return Agent;
}());
exports.Agent = Agent;
