"use strict";
exports.__esModule = true;
exports.Environment = void 0;
var userAgent_1 = require("./utils/userAgent");
var path_1 = require("path");
var fs_1 = require("fs");
var Environment = (function () {
    function Environment() {
        this.referrerUrl = "";
        this.start = new Date();
        this.url = "";
        this.userAgent = userAgent_1.userAgent;
    }
    Environment.prototype.clone = function () {
        return Object.assign(new Environment(), this);
    };
    Environment.prototype.getDependencies = function () {
        if (!Environment.dependencyCache) {
            this.discoverDependencies();
        }
        return Object.assign({}, Environment.dependencyCache);
    };
    Environment.prototype.discoverDependencies = function (_bustCache) {
        if (Environment.dependencyCache && !_bustCache) {
            return;
        }
        Environment.dependencyCache = {};
        var pathCache = {};
        var rootPaths = (require.main && require.main.paths) || [];
        var moduleFiles = require.cache ? Object.keys(require.cache) : [];
        function recurseUpDirTree(dir, roots, maxDepth) {
            if (roots === void 0) { roots = []; }
            if (maxDepth === void 0) { maxDepth = 10; }
            if (maxDepth === 0 || !dir || pathCache[dir] || roots.indexOf(dir) >= 0) {
                return;
            }
            pathCache[dir] = 1;
            var packageFile = (0, path_1.join)(dir, "package.json");
            if ((0, fs_1.existsSync)(packageFile)) {
                try {
                    var packageInfo = JSON.parse((0, fs_1.readFileSync)(packageFile, "utf8"));
                    Environment.dependencyCache[packageInfo.name] = packageInfo.version;
                }
                catch (err) {
                }
            }
            else {
                return recurseUpDirTree((0, path_1.dirname)(dir), roots, maxDepth--);
            }
        }
        moduleFiles.forEach(function (moduleFile) {
            var modulePath = (0, path_1.dirname)(moduleFile);
            recurseUpDirTree(modulePath, rootPaths);
        });
    };
    Environment.dependencyCache = null;
    return Environment;
}());
exports.Environment = Environment;
