"use strict";
exports.__esModule = true;
exports.Metadata = void 0;
var serialize_1 = require("./utils/serialize");
var isType_1 = require("./utils/isType");
var Metadata = (function () {
    function Metadata(initialMeta) {
        this._hash = {};
        if (initialMeta) {
            this.add(initialMeta);
        }
    }
    Metadata.prototype.add = function (meta, value) {
        var _this = this;
        if ((0, isType_1.isString)(meta)) {
            this._hash[meta] = value;
        }
        else {
            Object.keys(meta).forEach(function (key) {
                _this._hash[(0, serialize_1.serialize)(key)] = (0, serialize_1.serialize)(meta[key]);
            });
        }
    };
    Metadata.prototype.clone = function () {
        var cloned = new Metadata();
        cloned.add(this.get().reduce(function (pre, cur) {
            pre[cur.key] = cur.value;
            return pre;
        }, {}));
        return cloned;
    };
    Metadata.prototype.get = function () {
        var _this = this;
        return Object.keys(this._hash).map(function (key) {
            return { key: key, value: _this._hash[key] };
        });
    };
    Metadata.prototype.remove = function (meta) {
        var _this = this;
        if ((0, isType_1.isString)(meta)) {
            delete this._hash[meta];
        }
        else {
            Object.keys(meta).forEach(function (key) {
                delete _this._hash[(0, serialize_1.serialize)(key)];
            });
        }
    };
    return Metadata;
}());
exports.Metadata = Metadata;
