"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.expressErrorHandler = exports.expressRequestHandler = void 0;
var domain_1 = __importDefault(require("domain"));
var AgentRegistrar_1 = require("../AgentRegistrar");
var uuid_1 = require("../utils/uuid");
var TrackJSCapturePayload_1 = require("../types/TrackJSCapturePayload");
var DOMAIN_CARRIER_KEY = "__trackjs_domain__";
function getStatusCode(error) {
    var statusCode = error.status || error.statusCode || error.status_code || (error.output && error.output.statusCode);
    return statusCode ? parseInt(statusCode, 10) : 500;
}
function expressRequestHandler(options) {
    if (options === void 0) { options = { correlationHeader: true }; }
    return function trackjsExpressRequestHandler(req, res, next) {
        var requestDomain = domain_1["default"].create();
        requestDomain.add(req);
        requestDomain.add(res);
        requestDomain.on("error", next);
        requestDomain.run(function () {
            var agent = AgentRegistrar_1.AgentRegistrar.getCurrentAgent();
            req[DOMAIN_CARRIER_KEY] = requestDomain;
            var correlationId = (0, uuid_1.uuid)();
            agent.configure({ correlationId: correlationId });
            agent.environment.start = new Date();
            agent.environment.referrerUrl = req["headers"]["referer"] || "";
            agent.environment.url = "".concat(req["protocol"], "://").concat(req["get"]("host")).concat(req["originalUrl"]);
            if (req["headers"]["user-agent"]) {
                agent.metadata.add("__TRACKJS_REQUEST_USER_AGENT", req["headers"]["user-agent"]);
            }
            if (options.correlationHeader) {
                res.setHeader("TrackJS-Correlation-Id", correlationId);
                res.setHeader("Access-Control-Expose-Headers", "TrackJS-Correlation-Id");
            }
            next();
        });
    };
}
exports.expressRequestHandler = expressRequestHandler;
function expressErrorHandler(options) {
    if (options === void 0) { options = { next: false }; }
    return function trackjsExpressErrorHandler(error, req, res, next) {
        if (error && !error["__trackjs__"]) {
            var statusCode = getStatusCode(error);
            if (statusCode < 500) {
                next();
                return;
            }
            AgentRegistrar_1.AgentRegistrar.getCurrentAgent(req[DOMAIN_CARRIER_KEY]).captureError(error, TrackJSCapturePayload_1.TrackJSEntry.Express);
        }
        if (options.next) {
            next(error);
        }
    };
}
exports.expressErrorHandler = expressErrorHandler;
