"use strict";
exports.__esModule = true;
exports.TelemetryBuffer = void 0;
var TelemetryBuffer = (function () {
    function TelemetryBuffer(bufferSize, store) {
        this._size = bufferSize;
        this._store = store ? store.slice(0) : [];
    }
    TelemetryBuffer.prototype.add = function (category, data) {
        var key = Symbol();
        this._store.push({ key: key, category: category, data: data });
        if (this._store.length > this._size) {
            this._store = this._store.slice(Math.max(this._store.length - this._size, 0));
        }
        return key;
    };
    TelemetryBuffer.prototype.clear = function () {
        this._store.length = 0;
    };
    TelemetryBuffer.prototype.clone = function () {
        return new TelemetryBuffer(this._size, this._store);
    };
    TelemetryBuffer.prototype.count = function () {
        return this._store.length;
    };
    TelemetryBuffer.prototype.get = function (key) {
        var result = this._store.find(function (envelope) { return envelope.key === key; });
        return result ? result.data : null;
    };
    TelemetryBuffer.prototype.getAllByCategory = function (category) {
        return this._store.filter(function (envelope) { return envelope.category === category; }).map(function (envelope) { return envelope.data; });
    };
    return TelemetryBuffer;
}());
exports.TelemetryBuffer = TelemetryBuffer;
