"use strict";
exports.__esModule = true;
exports.ConsoleWatcher = void 0;
var patch_1 = require("../utils/patch");
var telemetry_1 = require("../telemetry");
var AgentRegistrar_1 = require("../AgentRegistrar");
var TrackJSCapturePayload_1 = require("../types/TrackJSCapturePayload");
var isType_1 = require("../utils/isType");
var CONSOLE_FN_NAMES = ["debug", "info", "warn", "error", "log"];
var _ConsoleWatcher = (function () {
    function _ConsoleWatcher() {
    }
    _ConsoleWatcher.prototype.install = function (_console) {
        var consoleObj = _console || console;
        CONSOLE_FN_NAMES.forEach(function (name) {
            (0, patch_1.patch)(consoleObj, name, function (originalFn) {
                return function () {
                    var messages = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        messages[_i] = arguments[_i];
                    }
                    var agent = AgentRegistrar_1.AgentRegistrar.getCurrentAgent();
                    var data = new telemetry_1.ConsoleTelemetry(name, messages);
                    agent.telemetry.add("c", data);
                    if (name === "error") {
                        if (messages.length === 1 && (0, isType_1.isError)(messages[0])) {
                            agent.captureError(messages[0], TrackJSCapturePayload_1.TrackJSEntry.Console);
                        }
                        else {
                            agent.captureError(new Error(data.message), TrackJSCapturePayload_1.TrackJSEntry.Console);
                        }
                    }
                    return originalFn.apply(consoleObj, arguments);
                };
            });
        });
    };
    _ConsoleWatcher.prototype.uninstall = function (_console) {
        var consoleObj = _console || console;
        CONSOLE_FN_NAMES.forEach(function (name) { return (0, patch_1.unpatch)(consoleObj, name); });
    };
    return _ConsoleWatcher;
}());
exports.ConsoleWatcher = new _ConsoleWatcher();
