"use strict";
exports.__esModule = true;
exports.NetworkWatcher = void 0;
var patch_1 = require("../utils/patch");
var telemetry_1 = require("../telemetry");
var AgentRegistrar_1 = require("../AgentRegistrar");
var TrackJSCapturePayload_1 = require("../types/TrackJSCapturePayload");
var HttpAgent = require("_http_agent");
var _NetworkWatcher = (function () {
    function _NetworkWatcher() {
    }
    _NetworkWatcher.prototype.install = function (_agent) {
        var agentPrototype = _agent ? _agent.prototype : HttpAgent.Agent.prototype;
        (0, patch_1.patch)(agentPrototype, "addRequest", function (originalAddRequest) {
            return function addRequest(req, options, port, localAddress) {
                if (!(options || {})["__trackjs__"]) {
                    var networkTelemetry = _NetworkWatcher.createTelemetryFromRequest(req);
                    AgentRegistrar_1.AgentRegistrar.getCurrentAgent(req["domain"]).telemetry.add("n", networkTelemetry);
                }
                return originalAddRequest.apply(this, arguments);
            };
        });
    };
    _NetworkWatcher.prototype.uninstall = function (_agent) {
        var agentPrototype = _agent ? _agent.prototype : HttpAgent.Agent.prototype;
        (0, patch_1.unpatch)(agentPrototype, "addRequest");
    };
    _NetworkWatcher.createTelemetryFromRequest = function (request) {
        var networkTelemetry = new telemetry_1.NetworkTelemetry();
        networkTelemetry.type = "http";
        networkTelemetry.startedOn = new Date().toISOString();
        networkTelemetry.method = request["method"];
        networkTelemetry.url = "".concat(request["agent"].protocol, "//").concat(request.getHeader("host")).concat(request.path);
        request.once("socket", function () {
            networkTelemetry.startedOn = new Date().toISOString();
        });
        request.once("response", function (response) {
            networkTelemetry.statusCode = response.statusCode;
            networkTelemetry.statusText = response.statusMessage;
            networkTelemetry.completedOn = new Date().toISOString();
            if (networkTelemetry.statusCode >= 400) {
                AgentRegistrar_1.AgentRegistrar.getCurrentAgent(request["domain"]).captureError(new Error("".concat(networkTelemetry.statusCode, " ").concat(networkTelemetry.statusText, ": ").concat(networkTelemetry.method, " ").concat(networkTelemetry.url)), TrackJSCapturePayload_1.TrackJSEntry.Network);
            }
        });
        return networkTelemetry;
    };
    return _NetworkWatcher;
}());
exports.NetworkWatcher = new _NetworkWatcher();
